package com.bigdata.wxappserver.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bigdata.wxappserver.dto.OrderDto;
import com.bigdata.wxappserver.entity.Goods;
import com.bigdata.wxappserver.entity.Order;
import com.bigdata.wxappserver.entity.User;
import com.bigdata.wxappserver.enums.OrderStatusEnum;
import com.bigdata.wxappserver.mapper.OrderMapper;
import com.bigdata.wxappserver.result.Result;
import com.bigdata.wxappserver.utils.WeChatPayUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created with IntelliJ IDEA.
 *
 * @Author: Cool
 * @Date: 2024/08/28/20:28
 * @Description:
 */
@Service
public class OrderService extends ServiceImpl<OrderMapper, Order> {
    @Autowired
    private WeChatPayUtil weChatPayUtil;
    @Autowired
    UserService userService;

    @Autowired
    GoodsService goodsService;

    @Transactional(rollbackFor = Exception.class)
    public JSONObject addOrUpdate(JSONObject jsonObject) {
        Order order = jsonObject.toJavaObject(Order.class);
        if (order == null) {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "参数错误");
        }
        if (!StringUtils.hasLength(order.getAddress())) {
            User user = userService.getById(order.getUserId());
            order.setAddress(user.getAddress());
        }
        // TODO 判空
        boolean success = saveOrUpdate(order);
        return new JSONObject().fluentPut("message", "success").fluentPut("success", success);
    }

    public JSONObject list(JSONObject jsonObject) {
        String openId = jsonObject.getString("openId");
        List<User> userList = userService.list();
        User user = userList.stream().filter(e -> Objects.equals(e.getOpenId(), openId)).findFirst().orElse(null);
        Integer status = jsonObject.getInteger("status");
//        Integer currentPage = jsonObject.getInteger("currentPage");
//        Integer pageSize = jsonObject.getInteger("pageSize");
//        IPage<Order> page = new Page<>(currentPage, pageSize);
        LambdaQueryWrapper<Order> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(user != null, Order::getUserId, user.getId())
                .eq(status != null, Order::getStatus, status);
//        IPage<Order> orderIPage = page(page, wrapper);
        List<Order> list = list(wrapper);
        List<OrderDto> orderDtoList = dealList(list);
        JSONObject returnData = new JSONObject();
        returnData.fluentPut("data", orderDtoList);
        return returnData;
    }

    private List<OrderDto> dealList(List<Order> records) {
        if (CollectionUtils.isEmpty(records)) {
            return new ArrayList<>();
        }
        List<Goods> goodsList = goodsService.listByIds(records.stream().map(Order::getGoodsId).collect(Collectors.toSet()));
        Set<Goods> goodsSet = new HashSet<>(goodsList);
        List<OrderDto> list = new ArrayList<>();
        for (Order record : records) {
            OrderDto orderDto = new OrderDto();
            record.setStatusDescription(OrderStatusEnum.describe(record.getStatus()));
            BeanUtils.copyProperties(record, orderDto);
            Goods goods = goodsSet.stream().filter(item -> Objects.equals(item.getId(), record.getGoodsId()))
                    .findFirst().orElse(null);
            if (goods != null) {
                orderDto.setGoodDetail(goods.getGoodDetail());
                orderDto.setGoodPrice(goods.getGoodPrice());
                orderDto.setGoodImage(goods.getGoodImage());
                orderDto.setGoodName(goods.getGoodName());
            }
            list.add(orderDto);
        }
        return list;
    }


    public JSONObject queryById(Integer id) {
        if (id == null) {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "缺少ID");
        }
        Order order = getById(id);
        List<OrderDto> orderDtoList = dealList(Collections.singletonList(order));
        return new JSONObject().fluentPut("data", orderDtoList.get(0));
    }


    @Transactional(rollbackFor = Exception.class)
    public JSONObject delete(Integer id) {
        if (id == null) {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "缺少Id");
        }
        boolean success = removeById(id);
        if (success) {
            return new JSONObject().fluentPut("success", true);
        } else {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "出现未知错误，请联系管理员！");
        }
    }

    public void paySuccess(String outTradeNo) {
        List<Order> list = list();
        Order order = list.stream().filter(e -> Objects.equals(String.valueOf(e.getId()), outTradeNo)).findFirst().orElse(null);
        if (order != null) {
            order.setStatus(2);
        }
        saveOrUpdate(order);
    }

    /**
     * 订单支付
     *
     * @param ordersPaymentDTO
     * @return
     */
    public Result payment(JSONObject ordersPaymentDTO) throws Exception {
        // 当前登录用户id
        String openId = ordersPaymentDTO.getString("openId");
        String orderCode = ordersPaymentDTO.getString("orderCode");
        List<User> userList = userService.list();
        User user = userList.stream().filter(e -> Objects.equals(e.getOpenId(), openId)).findFirst().orElse(null);
        if (user == null) {
            return Result.error("查询不到该用户");
        }
        //调用微信支付接口，生成预支付交易单
        JSONObject jsonObject = weChatPayUtil.pay(
                orderCode, //商户订单号
                new BigDecimal(0.01), //支付金额，单位 元
                "Test123", //商品描述
                user.getOpenId() //微信用户的openid
        );

        if (jsonObject.getString("code") != null && jsonObject.getString("code").equals("ORDERPAID")) {
            return Result.error("该订单已支付");
        }

//        OrderPaymentVO vo = jsonObject.toJavaObject(OrderPaymentVO.class);
//        vo.setPackageStr(jsonObject.getString("package"));
//
//        return vo;
        return Result.success();
    }


}
