package com.bigdata.wxappserver.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bigdata.wxappserver.entity.User;
import com.bigdata.wxappserver.mapper.UserMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
public class UserService extends ServiceImpl<UserMapper, User> {
    private static final String UPLOAD_DIR = "uploads/";

    @Transactional(rollbackFor = Exception.class)
    public JSONObject addOrUpdate(JSONObject jsonObject) {
        User user = jsonObject.toJavaObject(User.class);
        if (user == null) {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "参数错误");
        }
        List<User> userList = list();
        User getUserIdByopenId = userList.stream().filter(e -> Objects.equals(e.getOpenId(), user.getOpenId())).findFirst().orElse(null);
        user.setId(getUserIdByopenId != null ? getUserIdByopenId.getId() : null);
        List<String> openIdList = userList.stream().map(User::getOpenId).collect(Collectors.toList());
        if (user.getId() == null && !CollectionUtils.isEmpty(openIdList) && openIdList.contains(user.getOpenId())) {
            return new JSONObject().fluentPut("message", "该用户已存在").fluentPut("success", false);
        }
        // TODO 判空
        boolean success = saveOrUpdate(user);
        return new JSONObject().fluentPut("success", success);
    }

    public JSONObject list(JSONObject jsonObject) {
        String userId = jsonObject.getString("userId");
        Integer currentPage = jsonObject.getInteger("currentPage");
        Integer pageSize = jsonObject.getInteger("pageSize");
        IPage<User> page = new Page<>(currentPage, pageSize);
        LambdaQueryWrapper<User> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(userId != null, User::getId, userId);
        IPage<User> userIPage = page(page, wrapper);
        dealList(userIPage.getRecords());
        JSONObject returnData = new JSONObject();
        returnData.fluentPut("data", userIPage.getRecords())
                .fluentPut("total", userIPage.getTotal())
                .fluentPut("current", userIPage.getCurrent());
        return returnData;
    }

    private void dealList(List<User> records) {
        if (CollectionUtils.isEmpty(records)) {
            return;
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public JSONObject delete(Integer id) {
        if (id == null) {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "缺少Id");
        }
        boolean success = removeById(id);
        if (success) {
            return new JSONObject().fluentPut("success", true);
        } else {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "出现未知错误，请联系管理员！");
        }
    }

    public JSONObject queryById(String openId) {
        if (openId == null) {
            return new JSONObject().fluentPut("success", false).fluentPut("message", "缺少openID");
        }

        List<User> userList = list();
        User user = userList.stream().filter(e -> Objects.equals(e.getOpenId(), openId)).findFirst().orElse(null);
        return new JSONObject().fluentPut("data", user);
    }

    public JSONObject upload(MultipartFile file) {
        // 检查上传文件是否为空
        if (file.isEmpty()) {
            return new JSONObject().fluentPut("message", "上传文件为空").fluentPut("success", false);
        }
        try {
            // 创建上传目录
            File directory = new File(UPLOAD_DIR);
            if (!directory.exists()) {
                directory.mkdirs();
            }
            // 保存文件到服务器
            String fileName = System.currentTimeMillis() + "_" + file.getOriginalFilename();
            Path path = Paths.get(UPLOAD_DIR + fileName);
            Files.write(path, file.getBytes());
            // 构建文件的访问URL
            String fileDownloadUri = ServletUriComponentsBuilder.fromCurrentContextPath()
                    .path("/" + UPLOAD_DIR)
                    .path(fileName)
                    .toUriString();
            return new JSONObject().fluentPut("success", true).fluentPut("fileDownloadUri", fileDownloadUri);
        } catch (IOException e) {
            e.printStackTrace();
            return new JSONObject().fluentPut("success", false).fluentPut("message", "上传文件失败");
        }
    }
}
